@@ -22,6 +22,8 @@
 #' @param k The number of clusters.
 #' @param type The measure which is used to compute the dissimilarity matrix. The preferred options
 #' are \code{"area"} and \code{"st"}, but \code{"erl"} and \code{"cont"} can be also used with caution.
+#' @param triangineq Logical. Whether or not to compute the proportion of combinations
+#' of functions which satisfies the triangular inequality, see 'Value'.
 #' @param ... Additional parameters to be passed to \code{\link{central_region}},
 #' which is responsible for calculating the central region (global envelope)
 #' on which the functional clustering is based.
@@ -90,7 +92,7 @@
 #' @importFrom stats dist
 #' @importFrom cluster pam
 #' @importFrom utils combn
-fclustering <- function(curve_sets, k, type = c("area", "st", "erl", "cont"), ...) {
+fclustering <- function(curve_sets, k, type = c("area", "st", "erl", "cont"), triangineq = FALSE, ...) {
   # Check curve_sets. Note: k is checked by pam
   if(is_a_single_curveset(curve_sets)) {
     curve_sets <- list(curve_sets) # Make a list of a single curve_set to treat it similarly as several sets of curves
@@ -160,18 +162,20 @@ fclustering <- function(curve_sets, k, type = c("area", "st", "erl", "cont"), ..
   bb <- as.matrix(b)
   nfunc <- curve_set_nfunc(curve_sets[[1]])
   # Triangular inequality
-  r <- NULL # Here r is a logical vector for something
-  for(i in 1:(nfunc-2)) {
-    for(j in (i+1):(nfunc-1)) {
-      for(kk in (j+1):(nfunc)) {
-        r <- c(r, bb[i,j] + bb[j,kk] >= bb[i,kk] - 0.00000001)
+  if(triangineq) {
+    r <- NULL # Here r is a logical vector for something
+    for(i in 1:(nfunc-2)) {
+      for(j in (i+1):(nfunc-1)) {
+        for(kk in (j+1):(nfunc)) {
+          r <- c(r, bb[i,j] + bb[j,kk] >= bb[i,kk] - 0.00000001)
+        }
       }
     }
   }
 
   res <- list(curve_sets=curve_sets, k=k, type=type,
-              pam=resultpamF, dis=b,
-              triangineq = sum(r)/length(r))
+              pam=resultpamF, dis=b)
+  if(triangineq) res$triangineq <- sum(r)/length(r)
   class(res) <- c("fclust", class(res))
   res
 }
